@@ -1075,6 +1075,8 @@ create_context (CoglDisplay *display, CoglError **error)
   COGL_NOTE (WINSYS, "Creating GLX Context (display: %p)",
              xlib_renderer->xdpy);
 
+  _cogl_xlib_renderer_trap_errors (display->renderer, &old_state);
+
   if (display->renderer->driver == COGL_DRIVER_GL3)
     glx_display->glx_context = create_gl3_context (display, config);
   else
@@ -1085,7 +1087,8 @@ create_context (CoglDisplay *display, CoglError **error)
                                          NULL,
                                          True);
 
-  if (glx_display->glx_context == NULL)
+  if (_cogl_xlib_renderer_untrap_errors (display->renderer, &old_state) ||
+      glx_display->glx_context == NULL)
     {
       _cogl_set_error (error, COGL_WINSYS_ERROR,
                    COGL_WINSYS_ERROR_CREATE_CONTEXT,
@@ -1159,7 +1162,7 @@ create_context (CoglDisplay *display, CoglError **error)
                                        dummy_drawable,
                                        glx_display->glx_context);
 
-  XFree (xvisinfo);
+  xlib_renderer->xvisinfo = xvisinfo;
 
   if (_cogl_xlib_renderer_untrap_errors (display->renderer, &old_state))
     {
@@ -2078,29 +2081,6 @@ _cogl_winsys_onscreen_set_resizable (CoglOnscreen *onscreen,
   XFree (size_hints);
 }
 
-/* XXX: This is a particularly hacky _cogl_winsys interface... */
-static XVisualInfo *
-_cogl_winsys_xlib_get_visual_info (void)
-{
-  CoglGLXDisplay *glx_display;
-  CoglXlibRenderer *xlib_renderer;
-  CoglGLXRenderer *glx_renderer;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
-
-  _COGL_RETURN_VAL_IF_FAIL (ctx->display->winsys, FALSE);
-
-  glx_display = ctx->display->winsys;
-  xlib_renderer = _cogl_xlib_renderer_get_data (ctx->display->renderer);
-  glx_renderer = ctx->display->renderer->winsys;
-
-  if (!glx_display->found_fbconfig)
-    return NULL;
-
-  return glx_renderer->glXGetVisualFromFBConfig (xlib_renderer->xdpy,
-                                                 glx_display->fbconfig);
-}
-
 static CoglBool
 get_fbconfig_for_depth (CoglContext *context,
                         unsigned int depth,
@@ -2714,7 +2694,6 @@ static CoglWinsysVtable _cogl_winsys_vtable =
     .context_init = _cogl_winsys_context_init,
     .context_deinit = _cogl_winsys_context_deinit,
     .context_get_clock_time = _cogl_winsys_get_clock_time,
-    .xlib_get_visual_info = _cogl_winsys_xlib_get_visual_info,
     .onscreen_init = _cogl_winsys_onscreen_init,
     .onscreen_deinit = _cogl_winsys_onscreen_deinit,
     .onscreen_bind = _cogl_winsys_onscreen_bind,
